@@ -21,14 +21,15 @@ It can be represented by the unitary matrix
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Optional
+from dataclasses import replace
+from typing import TYPE_CHECKING
 
 import numpy as np
 
 from exa.common.data.parameter import Parameter, Setting
 from exa.common.qcm_data.chip_topology import DEFAULT_2QB_MAPPING
 from iqm.pulse.gate_implementation import GateImplementation, Locus, OILCalibrationData, get_waveform_parameters
-from iqm.pulse.playlist.instructions import Block, Instruction, IQPulse, RealPulse, VirtualRZ
+from iqm.pulse.playlist.instructions import Block, FluxPulse, Instruction, IQPulse, VirtualRZ
 from iqm.pulse.playlist.schedule import Schedule
 from iqm.pulse.playlist.waveforms import (
     CosineRiseFall,
@@ -38,6 +39,7 @@ from iqm.pulse.playlist.waveforms import (
     TruncatedGaussianSmoothedSquare,
     Waveform,
 )
+from iqm.pulse.timebox import TimeBox
 from iqm.pulse.utils import phase_transformation
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -73,9 +75,9 @@ class FluxPulseGate(GateImplementation):
 
     """
 
-    coupler_wave: Optional[type[Waveform]]
+    coupler_wave: type[Waveform] | None
     """Flux pulse Waveform to be played in the coupler flux AWG."""
-    qubit_wave: Optional[type[Waveform]]
+    qubit_wave: type[Waveform] | None
     """Flux pulse Waveform to be played in the qubit flux AWG."""
     root_parameters: dict[str, Parameter | Setting] = {
         "duration": Parameter("", "Gate duration", "s"),
@@ -98,7 +100,6 @@ class FluxPulseGate(GateImplementation):
         builder: ScheduleBuilder,
     ):
         super().__init__(parent, name, locus, calibration_data, builder)
-
         duration = calibration_data["duration"]
         flux_pulses = {}
 
@@ -115,7 +116,7 @@ class FluxPulseGate(GateImplementation):
             n_samples = builder.channels[flux_channel].duration_to_int_samples(duration) if duration > 0 else 0
             params["n_samples"] = n_samples
             amplitude = params.pop("amplitude")
-            flux_pulses[flux_channel] = RealPulse(
+            flux_pulses[flux_channel] = FluxPulse(
                 duration=n_samples,
                 wave=waveform_class(**params),
                 scale=amplitude,
@@ -138,28 +139,29 @@ class FluxPulseGate(GateImplementation):
                 raise ValueError(
                     f"{parent.name}.{name}: {locus}: Calibration is missing an RZ angle for locus component {c}."
                 )
-        rz = {builder.get_drive_channel(c): angle for c, angle in rz.items()}
-
+        rz_locus = {builder.get_drive_channel(c): angle for c, angle in rz.items() if c in locus}
+        rz_not_locus = tuple((builder.get_drive_channel(c), angle) for c, angle in rz.items() if c not in locus)
         # No driving must happen on any of the affected components during the flux pulses,
         # hence the virtual z rotations must use up their entire duration.
         T = max(pulse.duration for pulse in flux_pulses.values())
         # The gate takes no parameters, so we may build and cache the entire Schedule here.
         schedule: dict[str, list[Instruction]] = {}
-        for channel, angle in rz.items():
+        for channel, angle in rz_locus.items():
             # the virtual rz technique requires decrementing the drive phase by the rz angle
             schedule[channel] = [VirtualRZ(duration=T, phase_increment=-angle)]
-
+        vzs_inserted = False  # insert the long-distance Vzs to the first flux pulse (whatever that is)
         for channel, flux_pulse in flux_pulses.items():
-            schedule[channel] = [flux_pulse]
+            if rz_not_locus and not vzs_inserted:
+                schedule[channel] = [replace(flux_pulse, rzs=rz_not_locus)]
+                vzs_inserted = True
+            else:
+                schedule[channel] = [flux_pulse]
         affected_components = set(locus)
         affected_components.add(builder.chip_topology.get_coupler_for(*locus))
         self._affected_components = affected_components
-
         self._schedule = Schedule(schedule if T > 0 else {c: [Block(0)] for c in schedule}, duration=T)
 
-    def __init_subclass__(
-        cls, /, coupler_wave: Optional[type[Waveform]] = None, qubit_wave: Optional[type[Waveform]] = None
-    ):
+    def __init_subclass__(cls, /, coupler_wave: type[Waveform] | None = None, qubit_wave: type[Waveform] | None = None):
         """Store the Waveform types used by this subclass, and their parameters.
 
         NOTE: if ``MyCZ`` is a subclass of ``FluxPulseGate``, with some defined coupler and qubit waves, further
@@ -257,9 +259,9 @@ class CouplerFluxPulseQubitACStarkPulseGate(GateImplementation):
 
     """
 
-    coupler_wave: Optional[type[Waveform]]
+    coupler_wave: type[Waveform] | None
     """Flux pulse Waveform to be played in the coupler flux AWG."""
-    qubit_drive_wave: Optional[type[Waveform]]
+    qubit_drive_wave: type[Waveform] | None
     """Qubit drive pulse waveform to be played in the qubit drive AWG."""
 
     root_parameters: dict[str, Parameter | Setting] = {
@@ -299,7 +301,7 @@ class CouplerFluxPulseQubitACStarkPulseGate(GateImplementation):
             n_samples = builder.channels[flux_channel].duration_to_int_samples(duration) if duration > 0 else 0
             params["n_samples"] = n_samples
             amplitude = params.pop("amplitude")
-            flux_pulses[flux_channel] = RealPulse(
+            flux_pulses[flux_channel] = FluxPulse(
                 duration=n_samples,
                 wave=waveform_class(**params),
                 scale=amplitude,
@@ -332,8 +334,12 @@ class CouplerFluxPulseQubitACStarkPulseGate(GateImplementation):
 
         for channel, qubit_drive_pulse in qubit_drive_pulses.items():
             schedule[channel] = [qubit_drive_pulse]
-        for channel, flux_pulse in flux_pulses.items():
-            schedule[channel] = [flux_pulse]
+        rz_not_locus = tuple((builder.get_drive_channel(c), angle) for c, angle in rz.items() if c not in locus)
+        for channel, flux_pulse in flux_pulses.items():  # just one flux pulse here
+            if rz_not_locus:
+                schedule[channel] = [replace(flux_pulse, rzs=rz_not_locus)]
+            else:
+                schedule[channel] = [flux_pulse]
 
         affected_components = set(locus)
         affected_components.add(builder.chip_topology.get_coupler_for(*locus))
@@ -342,7 +348,7 @@ class CouplerFluxPulseQubitACStarkPulseGate(GateImplementation):
         self._schedule = Schedule(schedule) if T > 0 else Schedule({c: [Block(0)] for c in schedule})
 
     def __init_subclass__(
-        cls, /, coupler_wave: Optional[type[Waveform]] = None, qubit_drive_wave: Optional[type[Waveform]] = None
+        cls, /, coupler_wave: type[Waveform] | None = None, qubit_drive_wave: type[Waveform] | None = None
     ):
         """Store the Waveform types used by this subclass, and their parameters."""
         cls.coupler_wave = coupler_wave
