@@ -34,8 +34,9 @@ The phase :math:`a` is canceled when the MOVE gate is applied a second time due
 
 from __future__ import annotations
 
+from collections.abc import Iterable
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, Iterable
+from typing import TYPE_CHECKING
 
 import numpy as np
 
